<?php
if (!function_exists('wp_rml_root_childs')) {
    /**
     * Gets the first level childs of the media library.
     * 
     * @return Array of RML_Folder objects
     */
    function wp_rml_root_childs() {
        return RML_Structure::getInstance()->getTree();
    }
}

if (!function_exists('wp_rml_select_tree')) {
    /**
     * Returns a .rml-root-list with an given tree. The selected folder id is
     * saved automatically in a hidden input type.
     * 
     * @param inputName the name for the hidden input type and the name for the list
     * @param selected the selected folder id (saved also in hidden input type)
     * @param tree Array of RML_Folder objects
     * @param extraClasses classes for the rml root list container
     * @return Formatted HTML string
     * 
     * Experimental:
     * <strong>Note #1</strong> The select tree has a javascript callback when it
     * is initalized. You can bind it with this snippet:
     * 
     * window.rml.hooks.register("customList", function(obj, $) {
     *       //if (obj.hasClass("my-extra-class")) {
     *            alert(obj.html());
     *       //}
     * });
     * 
     * <strong>Note #2</strong> If you want to use the select tree after a DOM change (ajax,
     * for example: Modal dialog in visual editor) please call the javascript function
     * window.rml.library.customLists() to affect the initalization referred to Note #1.
     * 
     * <strong>Note #3</strong> You can use a sub class of RML_Folder to customize your tree.
     * 
     * @see To see an demo how to use it, have a look at ../inc/admin_footer/sidebar.dummy.php:96
     * @see ../assets/js/library.js:customLists()
     * @see Filters are available for the TreeNode: RML_View
     */
    function wp_rml_select_tree($inputName, $selected, $tree, $extraClasses = "") {
        $output = '<div class="aio-tree rml-root-list rml-custom-list ' . $extraClasses . '" id="rml-list-' . $inputName . '" data-id="' . $inputName . '">
                <input type="hidden" name="' . $inputName . '" value="' . $selected . '" />
                
                <div class="aio-list-standard">
                    <div class="aio-nodes">
                        ' . RML_Structure::getInstance()->getView()->treeHTML($selected, $tree, $inputName) . '
                    </div>
                </div>
            </div>';
        return $output;
    }
}

if (!function_exists('wp_rml_create')) {
    /**
     * Creates a folder. At first it checks if a folder in parent already exists.
     * Then it checks if the given type is allowed in the parent.
     * 
     * @param $name String Name of the folder
     * @param $parent int ID of the parent
     * @param $type integer 0|1|2 @see Folder.class.inc
     * @param $restrictions Restrictions for this folder, see RML_Permissions
     * @param $supress_validation Supress the permission validation
     * @return  int (ID) when successfully => Check with is_numeric($result)
     *          array with error strings
     * @attention Use RML_Structure::getInstance()->resetData() to register the
     * new created folder/folders to the structure.
     */
    function wp_rml_create($name, $parent, $type, $restrictions = array(), $supress_validation = false) {
        return RML_Structure::getInstance()->createFolder($name, $parent, $type, $restrictions, $supress_validation);
    }
}

if (!function_exists('wp_rml_rename')) {
    /**
     * Renames a folder and then checks, if there is no duplicate folder in the
     * parent folder.
     * 
     * @param $name String New name of the folder
     * @param $id The ID of the folder
     * @param $supress_validation Supress the permission validation
     * @return true or Array with errors
     */
    function wp_rml_rename($name, $id, $supress_validation = false) {
        $folder = wp_rml_get_by_id($id, null, true);
        if ($folder !== null) {
            return $folder->setName($name, $supress_validation);
        }else{
            return array(__("The given folder does not exist or you can not rename this folder.", RML_TD));
        }
    }
}

if (!function_exists('wp_rml_delete')) {
    /**
     * Deletes a folder by ID.
     * 
     * @param $id The ID of the folder
     * @param $supress_validation Supress the permission validation
     * @return true or Array with errors
     */
    function wp_rml_delete($id, $supress_validation = false) {
        return RML_Structure::getInstance()->deleteFolder($id, $supress_validation);
    }
}

if (!function_exists('wp_rml_update_count')) {
    /**
     * Handle the count cache for the folders. This should avoid
     * a lack SQL subquery which loads data from the postmeta table.
     * 
     * @param $folders Array of folders ID, if null then all folders with cnt = NULL are updated
     * @param $attachments Array of attachments ID, is merged with $folders if given
     */
    function wp_rml_update_count($folders = null, $attachments = null) {
        RML_Structure::getInstance()->updateCountCache($folders, $attachments);
    }
}

if (!function_exists('wp_rml_dropdown')) {
    /**
     * This functions returns a HTML formatted string which contains
     * <options> elements with all folders, collections and galleries.
     * 
     * @param $selected The selected item
     *              "":             "All Files"
     *              _wp_rml_root(): "Root"
     *              int:            Folder ID
     * @param $disabled array Defines, which folder types are disabled (@see ./real-media-library.php for Constant-Types)
     *                        Default disabled is RML_TYPE_COLLECTION
     * @param $useAll boolean Defines, if "All Files" should be showed
     * @return String
     */
    function wp_rml_dropdown($selected, $disabled, $useAll = true) {
        return RML_Structure::getInstance()->optionsFasade($selected, $disabled, $useAll);
    }
}

if (!function_exists('wp_rml_dropdown_collection')) {
    /**
     * This functions returns a HTML formatted string which contains
     * <options> elements with all folders, collections and galleries.
     * Note: Only COLLECTIONS are SELECTABLE!
     * 
     * @param $selected The selected item
     *              "":             "All Files"
     *              _wp_rml_root(): "Root"
     *              int:            Folder ID
     * @return String
     */
    function wp_rml_dropdown_collection($selected) {
        return wp_rml_dropdown($selected, array(0,2,3,4));
    }
}

if (!function_exists('wp_rml_dropdown_gallery')) {
    /**
     * This functions returns a HTML formatted string which contains
     * <options> elements with all folders, collections and galleries.
     * Note: Only GALLERIES are SELECTABLE!
     * 
     * @param $selected The selected item
     *              "":             "All Files"
     *              _wp_rml_root(): "Root"
     *              int:            Folder ID
     * @return String
     */
    function wp_rml_dropdown_gallery($selected) {
        return wp_rml_dropdown($selected, array(0,1,3,4));
    }
}

if (!function_exists('wp_rml_dropdown_gallery_or_collection')) {
    /**
     * This functions returns a HTML formatted string which contains
     * <options> elements with all folders, collections and galleries.
     * Note: Only GALLERIES AND COLLECTIONS are SELECTABLE!
     * 
     * @param $selected The selected item
     *              "":             "All Files"
     *              _wp_rml_root(): "Root"
     *              int:            Folder ID
     * @return String
     */
    function wp_rml_dropdown_gallery_or_collection($selected) {
        return wp_rml_dropdown($selected, array(0,3,4));
    }
}

if (!function_exists('wp_rml_is_type')) {
    /**
     * Determines, if a Folder is a special folder type.
     * 
     * @param $folder RML_Folder or int
     * @param $allowed array Defines, which folder types are allowed (@see ./real-media-library.php for Constant-Types) 
     * @return boolean
     */
    function wp_rml_is_type($folder, $allowed) {
        if (!$folder instanceof RML_Folder) {
            $folder = wp_rml_get_by_id($folder, null, true);
            
            if (!$folder instanceof RML_Folder) {
                return false;
            }
        }
        
        return in_array($folder->type, $allowed);
    }
}

if (!function_exists('wp_rml_get_by_id')) {
    /**
     * This functions checks if a specific folder exists by ID and is
     * a given allowed RML Folder Type. If the given folder is _wp_rml_root() you will
     * get the first level folders.
     * 
     * @param $id int Folder ID
     * @param $allowed array Defines, which folder types are allowed (@see ./real-media-library.php for Constant-Types)
     *                       If it is null, all folder types are allowed.
     * @param $mustBeFolderObject Defines if the function may return the wp_rml_root_childs result
     * @return RML_Folder object or NULL
     * 
     * Note: The Folder ID must be a valid Folder ID, not Root and "All Files" => FolderID > _wp_rml_root()
     */
    function wp_rml_get_by_id($id, $allowed = null, $mustBeFolderObject = false) {
        if (!is_numeric($id)) {
            return null;
        }
        
        if ($mustBeFolderObject == false && $id == _wp_rml_root()) {
            return wp_rml_root_childs();
        }
        
        $folder = RML_Structure::getInstance()->getFolderByID($id);
        
        if (is_array($allowed)) {
            if (!wp_rml_is_type($folder, $allowed)) {
                return null;
            }
        }
        
        return $folder;
    }
}

if (!function_exists('wp_rml_get_by_absolute_path')) {
    /**
     * This functions checks if a specific folder exists by absolute path and is
     * a given allowed RML Folder Type.
     * 
     * @param $path string Folder Absolute Path
     * @param $allowed array Defines, which folder types are allowed (@see ./real-media-library.php for Constant-Types)
     *                       If it is null, all folder types are allowed.
     * @return RML_Folder object or NULL
     * 
     * Note: The absolute path may not be "/" (Root).
     */
    function wp_rml_get_by_absolute_path($path, $allowed = null) {
        $folder = RML_Structure::getInstance()->getFolderByAbsolutePath($path);
        
        if (is_array($allowed)) {
            if (!wp_rml_is_type($folder, $allowed)) {
                return null;
            }
        }
        
        return $folder;
    }
}

if (!function_exists('_wp_rml_root')) {
    /**
     * Get the parent root folder for a given blog id.
     * 
     * @experimental
     * @filter RML/ParentRoot
     * @example
        add_action("init", function() {
        	add_filter("RML/ParentRoot", function($root, $blogId) {
        		$current_user = wp_get_current_user();
        		return $current_user->ID;
        	}, 10, 2);
        }, 1);
     */
    function _wp_rml_root() {
        $result = apply_filters("RML/ParentRoot", -1, get_current_blog_id());
        return $result;
    }
}