<?php
/**
 * This class handles all hooks for future updates (migrations).
 */

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class RML_Migration {
    private static $me = null;
    
    private function __construct() {
        // Silence is golden.
    }
    
    /**
     * @hooked plugins_loaded
     */
    public function plugins_loaded() {
        add_action("RML/Sidebar/Content",           array($this, "sidebar_content"));
        add_action("RML/Options/Register",          array($this, "register_options_07102016"));
        add_filter("RML/Migration/07102016",        array($this, "migrate_07102016"));
    }
    
    /**
     * ======================================================
     * MIGRATION BUILD FROM:    <= 2.6.3 to >= 2.6.4
     *          BUILD DATE:     07102016
     * ======================================================
     * 
     * @param 
     * @hooked RML/Sidebar/Content
     * @migration 07102016
     */
    public function sidebar_content($folders) {
        $migrations = $this->getMigrations();
        $build = "07102016";
        
        if (isset($migrations[$build])
            && $migrations[$build][2]) {
            
            if ($folders->getCntAttachments() > 0) {
                echo '<div class="notice inline notice-warning notice-alt">
                    <p>
                        Uaaargh! Your image relationships can be broken now! It is because RML has some technical changes in the new version 2.6.4.
                        Go to <strong>Media Settings > RML - "Reset"</strong> and import the old relationships.
                        <a href="' . admin_url("options-media.php") . '">Goto Media settings</a> or 
                        <a href="#" class="rml-migration-dismiss" data-build="' . $build . '">dismiss this notice</a>.
                    </p>
                </div>';
            }else if (!is_multisite()) {
                // Dismiss
                $migrations[$build][2] = false;
                $this->updateMigrations($migrations);
            }
        }
    }
    
    /**
     * Do the migration if no multisite.
     */
    public function migrate_07102016($arr) {
        if (!is_multisite()) {
            $this->do_07102016();
            $arr[2] = false;
        }
        return $arr;
    }
    
    /**
     * Do the migration of the current blog id.
     */
    public function do_07102016() {
        global $wpdb;
        
        $sql = 'INSERT IGNORE INTO ' . RML_Core::getInstance()->getTableName("posts") . ' (`attachment`, `fid`, `isShortcut`, `bid`)
        SELECT wpp.ID AS attachment, rml.id AS fid, 0 AS isShortcut, rml.bid AS bid
        FROM ' . $wpdb->postmeta . ' AS wpm
        INNER JOIN ' . $wpdb->posts . ' AS wpp ON wpp.ID = wpm.post_id
        LEFT JOIN ' . RML_Core::getInstance()->getTableName() . ' AS rml ON rml.id = wpm.meta_value
        WHERE wpm.meta_key = "_rml_folder"
        AND rml.id > 0
        ON DUPLICATE KEY UPDATE fid=VALUES(fid), isShortcut=VALUES(isShortcut);';
        $wpdb->query($sql);
        RML_Structure::getInstance()->resetCountCache();
    }
    
    /**
     * Register a migration button to copy the files from wp_postmeta
     * to wp_realmedialibrary_posts table.
     * 
     * @hooked RML/Options/Register
     * @see this::do_07102016
     */
    public function register_options_07102016() {
        $migrations = $this->getMigrations();
        $build = "07102016";
        
        if (isset($migrations[$build])) {
            add_settings_field(
                'rml_button_migrate_07102016',
                '<label>'.__('Migration <= 2.6.3' , RML_TD ).'</label>',
                array($this, 'html_rml_button_07102016'),
                'media',
                'rml_options_migration'
            );
        }
    }
    
    public function html_rml_button_07102016() {
        echo '<button class="rml-button-wipe button" data-nonce-key="migrateDismiss" data-action="rml_migration" data-method="07102016">' . __('Import image relationships', RML_TD) . '</button>
            <p class="description">' . __("You have used an old version (<= 2.6.3) of RML and the new version (>= 2.6.4) has changed the place where the image relationships are stored.", RML_TD) . '</p>';
    }
    
    /**
     * ======================================================
     * CREATE MIGRATIONS
     * 
     * Determines in a new update, if there is a migration available.
     * 
     * Array of migrations: {
     *    Idx:  Build date
     *      [0] => $old version
     *      [1] => $new version
     *      [2] => Boolean if migration is active and not finished, yet
     * }
     * @hooked RML/Migration
     */
    public function migration($old, $new) {
        if (!is_string($old)) {
            return;
        }
        
        $migrations = $this->getMigrations();
        
        /**
         * Old:     <= 2.6.3
         * New:     >= 2.6.4
         * Then:    Migrate the meta key values to the new table
         */
        if (version_compare($old, "2.6.3", "<=") && version_compare($new, "2.6.4", ">=")) {
            $migrations["07102016"] = $this->doMigration("07102016", "2.6.3", "2.6.4");
        }
        
        $this->updateMigration($migrations);
    }
    
    /**
     * @see this::migration
     */
    private function doMigration($build, $from, $to) {
        return apply_filters("RML/Migration/" . $build, array($from, $to, true));
    }
    
    /**
     * Dismiss an update notice.
     * 
     * @param $build The build version of the update
     * @return boolean
     */
    public function dismiss($build) {
        $migrations = $this->getMigrations();
        if (isset($migrations[$build])
            && $migrations[$build][2]) {
                $migrations[$build][2] = false;
                $this->updateMigration($migrations);
                return true;
        }else{
            return false;
        }
    }
    
    public function getMigrations() {
        return get_site_option("rml_migration", array());
    }
    
    public function updateMigration($migrations) {
        update_site_option("rml_migration", $migrations);
    }

    public static function getInstance() {
        if (self::$me == null) {
            self::$me = new RML_Migration();
        }
        return self::$me;
    }
}